/**
 * jq.web.carousel - a carousel library for html5 mobile apps
 * @copyright AppMobi 2011 - AppMobi
 *
 */
(function($) {
    var cache = [];
    $.fn.carousel = function(opts) {
        if (opts === undefined && this.length > 0)
        {
            return cache[this[0].id] ? cache[this[0].id] : null;
        }
        var tmp;
        for (var i = 0; i < this.length; i++) {
            tmp = new carousel(this[i], opts);
            if (this[i].id)
                cache[this[i].id] = tmp;
        }
        return this.length === 1 ? tmp : this;
    };

    var carousel = (function() {
        if (!window.WebKitCSSMatrix) {
            return;
        }
        var translateOpen = 'm11' in new WebKitCSSMatrix() ? "3d(" : "(";
        var translateClose = 'm11' in new WebKitCSSMatrix() ? ",0)" : ")";

        var carousel = function(containerEl, opts) {
            if (typeof containerEl === "string" || containerEl instanceof String) {
                this.container = document.getElementById(containerEl);
            } else {
                this.container = containerEl;
            }
            if (!this.container) {
                alert("Error finding container for carousel " + containerEl);
                return;
            }
            if (this instanceof carousel) {
                for (var j in opts) {
                    if (opts.hasOwnProperty(j)) {
                        this[j] = opts[j];
                    }
                }
            } else {

                return new carousel(containerEl, opts);
            }
            try {
                var that = this;
                this.pagingDiv = this.pagingDiv ? document.getElementById(this.pagingDiv) : null;


                // initial setup
                this.container.style.overflow = "hidden";
                this.container.style['-webkit-box-orient'] = "vertical";
                this.container.style['display'] = "-webkit-box";
                this.container.style['-webkit-box-orient'] = "vertical";
                if (this.vertical) {
                    this.horizontal = false;
                }
                var tmpHTML = this.container.innerHTML;
                //this.container.innerHTML = "";
                var el = document.createElement("div");

                var arr = Array.prototype.slice.call(this.container.childNodes);

                if (this.horizontal) {
                    el.style.display = "-webkit-box";
                    el.style['-webkit-box-flex'] = 1;
                }
                else {
                    el.style.display = "block";
                }
                $(el).append(arr);
                this.container.appendChild(el);

                this.el = el;
                this.refreshItems();
                el.addEventListener('touchmove', function(e) {
                    that.touchMove(e);
                }, false);
                el.addEventListener('touchend', function(e) {
                    that.touchEnd(e);
                }, false);
                el.addEventListener('touchstart', function(e) {
                    that.touchStart(e);
                }, false);
                var that = this;

                window.addEventListener("orientationchange", function() {
                    that.onMoveIndex(that.carouselIndex,0);
                }, false);

            } catch (e) {
                console.log("error adding carousel " + e);
            }
        };

        carousel.prototype = {
            startX: 0,
            startY: 0,
            dx: 0,
            dy: 0,
            screen_all:false,
            auto_play:false,
            right_down:false,
            myDivWidth: 0,
            myDivHeight: 0,
            cssMoveStart: 0,
            childrenCount: 0,
            carouselIndex: 0,
            vertical: false,
            horizontal: true,
            el: null,
            movingElement: false,
            container: null,
            pagingDiv: null,
            pagingCssName: "carousel_paging",
            pagingCssNameSelected: "carousel_paging_selected",
            pagingFunction: null,
            lockMove:false,
            auto_time:3000,
            // handle the moving function
            touchStart: function(e) {
                this.myDivWidth = numOnly(win_width);


                this.myDivHeight = numOnly(win_height);
                this.lockMove=false;
                if (e.touches[0].target && e.touches[0].target.type !== undefined) {
                    var tagname = e.touches[0].target.tagName.toLowerCase();
                    if (tagname === "select" || tagname === "input" || tagname === "button")  // stuff we need to allow
                    {
                        return;
                    }
                }
                if (e.touches.length === 1) {

                    this.movingElement = true;
                    this.startY = e.touches[0].pageY;
                    this.startX = e.touches[0].pageX;
                    var cssMatrix=$.getCssMatrix(this.el);

                    if (this.vertical) {
                        try {
                            this.cssMoveStart = numOnly(cssMatrix.f);
                        } catch (ex1) {
                            this.cssMoveStart = 0;
                        }
                    } else {
                        try {
                            this.cssMoveStart = numOnly(cssMatrix.e);
                        } catch (ex1) {
                            this.cssMoveStart = 0;
                        }
                    }
                }
            },
            touchMove: function(e) {
                //e.preventDefault();
                //e.stopPropagation();
                if(!this.movingElement)
                    return;
                if (e.touches.length > 1) {
                    return this.touchEnd(e);
                }

                var rawDelta = {
                    x: e.touches[0].pageX - this.startX,
                    y: e.touches[0].pageY - this.startY
                };

                if (this.vertical) {
                    var movePos = { x: 0, y: 0 };
                    this.dy = e.touches[0].pageY - this.startY;

                    this.dy += this.cssMoveStart;
                    movePos.y = this.dy;

                    e.preventDefault();
                    //e.stopPropagation();

                } else {
                    //iPhone double clicks workaround

                    if (!this.lockMove&&isHorizontalSwipe(rawDelta.x, rawDelta.y)) {
                        var movePos = {x: 0,y: 0};
                        this.dx = e.touches[0].pageX - this.startX;
                        this.dx += this.cssMoveStart;
                        e.preventDefault();
                        e.stopPropagation();  //zjj
                        movePos.x = this.dx;
                        //jq_s["gongyi_scroll"].lock();
                        if(this.right_down){
                            var ca_id=this.el.childNodes[this.carouselIndex*2+1].childNodes[1].id;
                            jq_s[ca_id].lock();
                        }

                    }
                    else
                        return this.lockMove=true;
                }

                var totalMoved = this.vertical ? ((this.dy % this.myDivHeight) / this.myDivHeight * 100) * -1 : ((this.dx % this.myDivWidth) / this.myDivWidth * 100) * -1; // get a percentage of movement.
                if(movePos.x>0) movePos.x=movePos.x/4; //zjj
                else if(movePos.x<-this.myDivWidth*(this.childrenCount-1)) movePos.x=movePos.x+((-movePos.x-this.myDivWidth*(this.childrenCount-1))*3/5); //zjj
                if(movePos)
                    this.moveCSS3(this.el, movePos);
            },
            touchEnd: function(e) {
                if(this.right_down){
                    var ca_id=this.el.childNodes[this.carouselIndex*2+1].childNodes[1].id;
                    jq_s[ca_id].unlock();
                }
                if (!this.movingElement) {
                    return;
                }

                e.preventDefault();
                //e.stopPropagation();

                var runFinal = false;
                try {
                    var cssMatrix=$.getCssMatrix(this.el);
                    var endPos = this.vertical ? numOnly(cssMatrix.f) : numOnly(cssMatrix.e);
                    var is_end=false;
                    if (endPos > 0) {
                        this.moveCSS3(this.el, {
                            x: 0,
                            y: 0
                        }, "300");
                        e.stopPropagation(); //zjj
                    } else {
                        var totalMoved = this.vertical ? ((this.dy % this.myDivHeight) / this.myDivHeight * 100) * -1 : ((this.dx % this.myDivWidth) / this.myDivWidth * 100) * -1; // get a percentage of movement.

                        if(totalMoved>2) e.stopPropagation();  //zjj
                        // Only need
                        // to drag 3% to trigger an event
                        var currInd = this.carouselIndex;

                        if (endPos < this.cssMoveStart && totalMoved > 3) {
                            currInd++; // move right/down
                        } else if ((endPos > this.cssMoveStart && totalMoved < 97)) {
                            currInd--; // move left/up
                        }
                        if (currInd > (this.childrenCount - 1)) {
                            is_end=true;
                            currInd = this.childrenCount - 1;
                        }
                        if (currInd < 0) {
                            currInd = 0;
                        }
                        var movePos = {
                            x: 0,
                            y: 0
                        };
                        if (this.vertical) {
                            movePos.y = (currInd * this.myDivHeight * -1);
                        }
                        else {
                            movePos.x = (currInd * this.myDivWidth * -1);
                        }



                        this.moveCSS3(this.el, movePos, "150");

                        if (this.pagingDiv && this.carouselIndex !== currInd) {
                            document.getElementById(this.container.id + "_" + this.carouselIndex).className = this.pagingCssName;
                            document.getElementById(this.container.id + "_" + currInd).className = this.pagingCssNameSelected;
                        }
                        if (this.carouselIndex != currInd)
                            runFinal = true;
                        this.carouselIndex = currInd;
                        if(this.right_down) index_tag(parseInt(currInd)+1);
                    }
                } catch (e) {
                    console.log(e);
                }
                this.dx = 0;
                this.movingElement = false;
                this.startX = 0;
                this.dy = 0;
                this.startY = 0;
                if (this.pagingFunction && typeof this.pagingFunction == "function")
                    this.pagingFunction(this.carouselIndex,is_end);

                if(this.container.id) img_on_load(this.container.id);



                var all_length=this.childrenCount;
                all_length=parseInt(all_length);
                //zjj 自动播放
                if(this.auto_play && all_length>1) {

                    var currInd = this.carouselIndex;

                    currInd=parseInt(currInd);

                    var this1=this;
                    if(this.carousel_settime) window.clearTimeout(this.carousel_settime);
                    this.carousel_settime=window.setTimeout(function(){




                        if (currInd == (all_length - 1)) {
                            currInd = 0;
                        }
                        else  {
                            currInd++;
                        }


                        if (this1.pagingDiv) {
                            document.getElementById(this1.container.id + "_" + this1.carouselIndex).className = this1.pagingCssName;
                            document.getElementById(this1.container.id + "_" + currInd).className = this1.pagingCssNameSelected;

                        }
                        this1.onMoveIndex(currInd);

                        //this.moveCSS3(this.el, movePos, "150");
                    },this.auto_time);

                }

            },
            onMoveIndex: function(newInd,transitionTime) {
                if(this.movingElement) return;
                this.carouselIndex=newInd;
                //zjj第4处需要修改
                this.myDivWidth = win_width;
                //this.myDivWidth = window.innerWidth;
                //this.myDivHeight = numOnly(this.container.clientHeight);
                var runFinal = false;
                try {

                    //document.getElementById(this.container.id + "_" + this.carouselIndex).className = this.pagingCssName;
                    var newTime = Math.abs(newInd - this.carouselIndex);

                    var ind = newInd;
                    if (ind < 0)
                        ind = 0;
                    if (ind > this.childrenCount - 1) {
                        ind = this.childrenCount - 1;
                    }
                    var movePos = {
                        x: 0,
                        y: 0
                    };
                    if (this.vertical) {
                        movePos.y = (ind * this.myDivHeight * -1);

                    }
                    else {
                        movePos.x = (ind * this.myDivWidth * -1);
                        if(movePos.x=="-320") movePos.x="-320";

                    }

                    var time =transitionTime?transitionTime: 50 + parseInt((newTime * 20));
                    this.moveCSS3(this.el, movePos, 250);


                    var all_length=this.childrenCount;
                    all_length=parseInt(all_length);
                    //zjj 自动播放
                    if(this.auto_play && all_length>1) {

                        var currInd = this.carouselIndex;

                        currInd=parseInt(currInd);

                        var this1=this;
                        if(this.carousel_settime) window.clearTimeout(this.carousel_settime);
                        this.carousel_settime=window.setTimeout(function(){




                            if (currInd == (all_length - 1)) {
                                currInd = 0;
                            }
                            else  {
                                currInd++;
                            }


                            if (this1.pagingDiv) {
                                document.getElementById(this1.container.id + "_" + this1.carouselIndex).className = this1.pagingCssName;
                                document.getElementById(this1.container.id + "_" + currInd).className = this1.pagingCssNameSelected;

                            }
                            this1.onMoveIndex(currInd);

                            //this.moveCSS3(this.el, movePos, "150");
                        },this.auto_time);

                    }

                    if (this.carouselIndex != ind)
                        runFinal = true;
                    this.carouselIndex = ind;
                    if (this.pagingDiv) {
                        //document.getElementById(this.container.id + "_" + this.carouselIndex).className = this.pagingCssName;
                        document.getElementById(this.container.id + "_" + this.carouselIndex).className = this.pagingCssNameSelected;

                    }
                } catch (e) {
                    console.log("Error " + e);
                }
                if (runFinal && this.pagingFunction && typeof this.pagingFunction == "function")
                    this.pagingFunction(currInd);
            },

            moveCSS3: function(el, distanceToMove, time, timingFunction) {
                if (!time)
                    time = 0;
                else
                    time = parseInt(time);
                if (!timingFunction)
                    timingFunction = "linear";

                el.style.webkitTransform = "translate" + translateOpen + distanceToMove.x + "px," + distanceToMove.y + "px" + translateClose;
                el.style.webkitTransitionDuration = time + "ms";
                el.style.webkitBackfaceVisiblity = "hidden";
                el.style.webkitTransformStyle = "preserve-3d";
                el.style.webkitTransitionTimingFunction = timingFunction;
            },

            addItem: function(el) {
                if (el && el.nodeType) {

                    this.container.childNodes[0].appendChild(el);
                    this.refreshItems();
                }
            },
            refreshItems: function() {
                var childrenCounter = 0;
                var that = this;
                var el = this.el;
                n = el.childNodes[0];
                var widthParam;
                var heightParam = "100%";
                var elems = [];
                for (; n; n = n.nextSibling) {
                    if (n.nodeType === 1) {
                        elems.push(n);
                        childrenCounter++;
                    }
                }
                var param = (100 / childrenCounter) + "%";
                this.childrenCount = childrenCounter;
                widthParam = parseFloat(100 / this.childrenCount) + "%";
                widthParam = parseFloat(100 / this.childrenCount) + "%";

                for (var i = 0; i < elems.length; i++) {
                    if (this.horizontal) {
                        elems[i].style.width = win_width+"px";
                        //elems[i].style.width = widthParam;
                        if(this.screen_all==true) elems[i].style.height = win_height+"px";
                        //else elems[i].style.height = "100%";
                    }
                    else {
                        elems[i].style.height = widthParam;
                        elems[i].style.width = "100%";
                        elems[i].style.display = "block";
                    }
                }
                this.moveCSS3(el, {
                    x: 0,
                    y: 0
                });
                if (this.horizontal) {
                    //el.style.width = Math.ceil((this.childrenCount) * 100) + "%";
                    el.style.width = Math.ceil((this.childrenCount) * win_width) + "px";
                    //console.log(el.style.width);
                    //el.style.height = "100%";
                    //el.style['min-height'] = "100%"
                }
                else {
                    el.style.width = "100%";
                    el.style.height = Math.ceil((this.childrenCount) * 100) + "%";
                    el.style['min-height'] = Math.ceil((this.childrenCount) * 100) + "%";
                }
                // Create the paging dots
                if (this.pagingDiv) {
                    this.pagingDiv.innerHTML = "";
                    for (i = 0; i < this.childrenCount; i++) {

                        var pagingEl = document.createElement("div");
                        pagingEl.id = this.container.id + "_" + i;
                        pagingEl.pageId = i;
                        if (i !== this.carouselIndex) {
                            pagingEl.className = this.pagingCssName;
                        }
                        else {
                            pagingEl.className = this.pagingCssNameSelected;
                        }
                        //pagingEl.onclick = function() {
                        //    that.onMoveIndex(this.pageId);
                        //};
                        var spacerEl = document.createElement("div");

                        spacerEl.style.width = "5px";
                        if(this.horizontal){
                            spacerEl.style.display = "inline-block";
                            spacerEl.innerHTML = "&nbsp;";
                        }
                        else{
                            spacerEl.innerHTML="&nbsp;";
                            spacerEl.style.display="block";
                        }

                        this.pagingDiv.appendChild(pagingEl);
                        if (i + 1 < (this.childrenCount))
                            this.pagingDiv.appendChild(spacerEl);
                        pagingEl = null;
                        spacerEl = null;
                    }
                    //this.pagingDiv.style.width = (this.childrenCount) * 15 + 15  + "px";
                    this.pagingDiv.style.height = "20px";
                }
                this.onMoveIndex(this.carouselIndex);

            }

        };
        return carousel;
    })();

    function isHorizontalSwipe(xAxis, yAxis) {
        var X = xAxis;
        var Y = yAxis;
        var Z = Math.round(Math.sqrt(Math.pow(X,2)+Math.pow(Y,2))); //the distance - rounded - in pixels
        var r = Math.atan2(Y,X); //angle in radians
        var swipeAngle = Math.round(r*180/Math.PI); //angle in degrees
        if ( swipeAngle < 0 ) { swipeAngle =  360 - Math.abs(swipeAngle); } // for negative degree values
        if (((swipeAngle <= 215) && (swipeAngle >= 155)) || ((swipeAngle <= 45) && (swipeAngle >= 0)) || ((swipeAngle <= 360) && (swipeAngle >= 315))) // horizontal angles with threshold
        {return true; }
        else {return false}
    }

})(jq);